package main

import (
	"encoding/json"
	"fmt"
	"io"
	"log"
	"os"
	"os/exec"
	"path/filepath"

	"github.com/gdamore/tcell/v2"
	"github.com/rivo/tview"
)

type Command struct {
	Name     string `json:"name"`
	Command string `json:"command"`
	Type     string `json:"type"`
}

func main() {
	app := tview.NewApplication()
	var commands []Command

	commands = getAllCommands()

	list := tview.NewList()

	deleteMode := false

	for j, cmd := range commands {
		commandId := j
		list.AddItem(cmd.Name, cmd.Command, rune(commandId), func() {
			if deleteMode {
				var newCommands []Command
				for i := 0; i < len(commands); i++ {
					if commands[i].Name != cmd.Name {
						newCommands = append(newCommands, commands[i])
					}
				}
				saveAllCommands(newCommands)
				list.SetTitle("Deleted, Restart to view changes...")
				return;
			}

			app.Stop()

			if commands[commandId].Type == "SSH" {
				fmt.Println(">> Connecting you to " + commands[commandId].Name)
			} else {
				fmt.Println(">> Executing " + commands[commandId].Name)
			}
			i := 0
			for i < len(commands[commandId].Command) && commands[commandId].Command[i] != ' ' {
				i++
			}
			if i + 1 == len(commands[commandId].Command) {
				execute(commands[commandId].Command)
			} else {
				execute(commands[commandId].Command[:i], commands[commandId].Command[i+1:])
			}
		})
	}
	list.SetBorder(true).SetTitle("Your Quick Commands").SetTitleAlign(tview.AlignCenter).SetBorderColor(tcell.Color129)
	flex := tview.NewFlex().AddItem(list, 0, 1, true)


	var newName string
	var newCommand string
	var newType string


	types := []string{"Command", "SSH"}

	form := tview.NewForm().
		AddInputField("Name", "", 20, nil, func(text string) {
			newName = text
		}).
		AddInputField("Command", "", 20, nil, func(text string) {
			newCommand = text
		}).
		AddDropDown("Type", types, 0, func(option string, optionIndex int) {
			newType = option
		})
	
	form.SetBorder(true).SetTitle("Create New Command").SetTitleAlign(tview.AlignCenter).SetBorderColor(tcell.Color129)

	pages := tview.NewPages().AddPage("defaultMenu", flex, true, true).AddPage("addForm", form, true, false)

	list.AddItem("Add Command", "", 'a', func() {
		pages.SwitchToPage("addForm")
	})
	
	list.AddItem("Toggle Delete", "", 'd', func() {
		if deleteMode {
			list.SetTitle("Your Quick Commmands")
			list.SetBorderColor(tcell.Color129)
			deleteMode = false
			} else {
				list.SetTitle("DELETE MODE")
				list.SetBorderColor(tcell.ColorOrangeRed)
				deleteMode = true
			}
		})
		
	list.AddItem("Quit", "", 'q', func() {
		app.Stop()
	})
		
	form.AddButton("Create", func ()  {
		if newName == "" || newCommand == "" || newType == "" {
			form.AddButton("Invalid Inputs", nil)
			return
		}
		newCommand := Command{
			Name: newName,
			Command: newCommand,
			Type: newType,
		}
		commands = append(commands, newCommand)

		saveAllCommands(commands)

		list := tview.NewList()

		for j, cmd := range commands {
			commandId := j
			list.AddItem(cmd.Name, cmd.Command, rune(commandId), func() {
				app.Stop()
				
				if commands[commandId].Type == "ssh" {
					fmt.Println(">> Connecting you to " + commands[commandId].Name)
				} else {
					fmt.Println(">> Executing " + commands[commandId].Name)
				}
				i := 0
				for i < len(commands[commandId].Command) && commands[commandId].Command[i] != ' ' {
					i++
				}
				if i == len(commands) {
					execute(commands[commandId].Command, "")
				} else {
					execute(commands[commandId].Command[:i], commands[commandId].Command[i+1:])
				}
			})
		}


		list.SetBorder(true).SetTitle("Your Quick Commands").SetTitleAlign(tview.AlignCenter).SetBorderColor(tview.Styles.TertiaryTextColor)

		newFlex := tview.NewFlex().AddItem(list, 0, 1, true)

		pages.AddAndSwitchToPage("defaultMenuNew", newFlex, true)
	})
	
	form.AddButton("Go Back", func ()  {
		pages.SwitchToPage("defaultMenu")
	})

	if err := app.SetRoot(pages, true).Run(); err != nil {
		panic(err)
	}
}


func execute(name string, args ...string) {
	// print command
	s := name
	if len(args) > 0 {
		for _, a := range args {
			s += " " + a
		}
	}
	fmt.Println(">", s)
	// start command
	cmd := exec.Command(name, args...)
	cmd.Env = os.Environ()
	cmd.Stdin = os.Stdin
	cmd.Stdout = os.Stdout
	err := cmd.Run()
	// print error
	if err != nil {
		fmt.Println("> error:", err.Error())
		fmt.Println("> press any key to continue")
	}
}

func getAllCommands() []Command {
	// filePath := "C:\\Users\\James M\\cmds\\.commands.json"
	// filePath := os.Getenv("COMMANDS_FILE_PATH")
	filePath := getCommandsPath()
	file, err := os.Open(filePath)
	if err != nil {
		log.Fatalln("Error opening command file:", err)
	}

	defer file.Close()

	data, err := io.ReadAll(file)
	if err != nil {
		log.Fatalln("Error reading command file:", err)
	}

	var commands []Command
	err = json.Unmarshal(data, &commands)
	if err != nil {
		log.Fatalln("Error decoding command file:", err)
	}
	return commands
}

func saveAllCommands(newCommands []Command) {
	data, err := json.Marshal(newCommands)
	if err != nil {
		log.Fatalln("Error decoding command file:", err)
	}

	// filePath := "C:\\Users\\James M\\cmds\\.commands.json"
	filePath := getCommandsPath()
	file, err := os.Open(filePath)
	if err != nil {
		log.Fatalln("Error opening command file:", err)
	}

	defer file.Close()

	
	fileErr := os.WriteFile(filePath, data, 0644)
	if fileErr != nil {
		log.Fatalln("Error writing to command file:", err)
	}
}

func getCommandsPath() string {
	log.Fatalln(os.Getenv("PATH"))
	userDir, err := os.UserHomeDir()
	if err != nil {
		log.Fatalln("Failed to get user home directory. Err:", err)
	}
	commandsPath := filepath.Join(userDir, ".commands.json")
	return commandsPath
}